﻿-- ********************************************************************************
-- Halo Minimalistic
-- A minimalistic halo range display.
-- By: Shenton
--
-- HaloMini.lua
-- ********************************************************************************

--[[-------------------------------------------------------------------------------
    Globals to locals
--]]-------------------------------------------------------------------------------

local _G = _G;
local print = print;
local pairs = pairs;
local select = select;
local IsItemInRange = IsItemInRange;
local CheckInteractDistance = CheckInteractDistance;
local UnitClass = UnitClass;
local UnitLevel = UnitLevel;
local UnitExists = UnitExists;
local UnitIsFriend = UnitIsFriend;
local SlashCmdList = SlashCmdList;
local GetLocale = GetLocale;
local GetScreenWidth = GetScreenWidth;
local GameTooltip = GameTooltip;
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory;

--[[-------------------------------------------------------------------------------
    Localization
--]]-------------------------------------------------------------------------------

local L = {};

-- à : \195\160    è : \195\168    ì : \195\172    ò : \195\178    ù : \195\185
-- á : \195\161    é : \195\169    í : \195\173    ó : \195\179    ú : \195\186
-- â : \195\162    ê : \195\170    î : \195\174    ô : \195\180    û : \195\187
-- ã : \195\163    ë : \195\171    ï : \195\175    õ : \195\181    ü : \195\188
-- ä : \195\164                    ñ : \195\177    ö : \195\182
-- æ : \195\166                                    ø : \195\184
-- ç : \195\167                                    œ : \197\147
-- Ä : \195\132    Ö : \195\150    Ü : \195\156    ß : \195\159

if ( GetLocale() == "frFR" ) then
    L["LOCKED"] = "Bloqu\195\169";
    L["UNLOCKED"] = "D\195\169bloqu\195\169";
    L["NOT_ON_PRIEST"] = "Pas sur un pr\195\170tre de niveau 90, tout \195\170 \195\169t\195\169 d\195\169sactiv\195\169. Vous devriez d\195\169sactiver Halo Minimalistic \195\160 l'\195\169cran de s\195\169lection des personnages.";
    L["OPTIONS_VERSION"] = "Version: %s";
    L["OPTIONS_LOCK_DESC"] = "Bloque ou d\195\169bloque Halo Minimalistic.";
    L["DATABASE_WIPED"] = "Base de donn\195\169es r\195\169initialis\195\169e avec ses valeurs par d\195\169faut.";
    L["SCALE"] = "Echelle";
    L["OPTIONS_SCALE_DESC"] = "D\195\169finir l'\195\169chelle (taille) de Halo Minimalistic.";
    L["REFRESH_RATE"] = "Taux de rafraichissement";
    L["OPTIONS_REFRESH_RATE_DESC"] = "D\195\169finir le taux de rafraichissement de Halo Minimalistic.";
    L["COMMAND_1"] = "Sans aucun argument cela bloque ou d\195\169bloque Halo Minimalistic.";
    L["COMMAND_2"] = "/hm config - Ouvre le panneau de configuration.";
    L["COMMAND_3"] = "/hm reset - Remet \195\160 z\195\169ro la position de Halo Minimalistic au centre de l'\195\169cran.";
    --L["COMMAND_4"] = "/hm wipedb - Purge la base de donnée de configuration.";
    L["RESET_POS"] = "Reset position";
    L["OPTIONS_RESET_POS_DESC"] = "Remet \195\160 z\195\169ro la position de Halo Minimalistic au centre de l'\195\169cran.";
    L["TOOLTIP_UNLOCKED_INFO"] = "Halo Minimalistic est d\195\169bloqu\195\169. La distance ne sera pas mise \195\160 jour tant que ce sera le cas.";
    L["SPEC_1"] = "Sp\195\169cialisation une";
    L["SPEC_2"] = "Sp\195\169cialisation deux";
    L["ENABLE_WITH_SPEC_1_DESC"] = "Activer avec la premi\195\168re sp\195\169cialisation (%s)";
    L["ENABLE_WITH_SPEC_2_DESC"] = "Activer avec la deuxi\195\168me sp\195\169cialisation (%s)";
    L["NONE"] = "Aucune";
    L["ENABLE"] = "Activer";
    L["MODE"] = "Mode";
    L["TARGET"] = "Cible";
    L["MOUSEOVER"] = "Survole";
    L["BOTH"] = "Les deux";
    L["MODE_DESC"] = [[    |cffc79c6eCible: |rHalo Minimalistic va v\195\169rifier la distance entre vous et votre cible.
    |cffc79c6eMouseover: |rHalo Minimalistic va v\195\169rifier la distance entre vous et l'unit\195\169 que vous survolez avec la souris.
    |cffc79c6eBoth: |rHalo Minimalistic va utiliser les deux m\195\169thodes, mais en pr\195\169f\195\169rant celle du survol.]];
else
    L["LOCKED"] = "Locked";
    L["UNLOCKED"] = "Unlocked";
    L["NOT_ON_PRIEST"] = "Not on a level 90 priest, disabling everything. You should disable Halo Minimalistic by the characters selection screen.";
    L["OPTIONS_VERSION"] = "Version: %s";
    L["OPTIONS_LOCK_DESC"] = "Lock or unlock the Halo Minimalistic frame.";
    L["DATABASE_WIPED"] = "Database wiped and set to default.";
    L["SCALE"] = "Scale";
    L["OPTIONS_SCALE_DESC"] = "Set the scale of the Halo Minimalistic frame.";
    L["REFRESH_RATE"] = "Refresh rate";
    L["OPTIONS_REFRESH_RATE_DESC"] = "Set the refresh rate of Halo Minimalistic.";
    L["COMMAND_1"] = "With no argument it will toggle lock/unlock.";
    L["COMMAND_2"] = "/hm config - Open the configuration panel.";
    L["COMMAND_3"] = "/hm reset - Reset frame position to the center of the screen.";
    --L["COMMAND_4"] = "/hm wipedb - Wipe options database.";
    L["RESET_POS"] = "Reset position";
    L["OPTIONS_RESET_POS_DESC"] = "Reset frame position to the center of the screen.";
    L["TOOLTIP_UNLOCKED_INFO"] = "Halo Minimalistic frame is unlocked. It will not update the target range until it is locked.";
    L["SPEC_1"] = "Specialization one";
    L["SPEC_2"] = "Specialization two";
    L["ENABLE_WITH_SPEC_1_DESC"] = "Enable with specialization one (%s)";
    L["ENABLE_WITH_SPEC_2_DESC"] = "Enable with specialization two (%s)";
    L["NONE"] = "None";
    L["ENABLE"] = "Enable";
    L["MODE"] = "Mode";
    L["TARGET"] = "Target";
    L["MOUSEOVER"] = "Mouseover";
    L["BOTH"] = "Both";
    L["MODE_DESC"] = [[    |cffc79c6eTarget: |rHalo Minimalistic will check range between you and your target.
    |cffc79c6eMouseover: |rHalo Minimalistic will check range between you and the unit your are mouse hovering.
    |cffc79c6eBoth: |rHalo Minimalistic will use both previous methods, but it will prefer the mouseover unit before your target.]];
end

--[[-------------------------------------------------------------------------------
    Main object
--]]-------------------------------------------------------------------------------

local A = CreateFrame("Frame");
A.L = L;
_G["HaloMinimalisticGlobal"] = A;

--[[-------------------------------------------------------------------------------
    Variables
--]]-------------------------------------------------------------------------------

A.version = GetAddOnMetadata("HaloMini", "Version");

A.defaultDB =
{
    point = "CENTER",
    relativeTo = "UIParent",
    relativePoint = "CENTER",
    offX = 0,
    offY = 0,
    locked = 1,
    scale = 1,
    refreshRate = 0.5,
    specOne = 1,
    specTwo = 1,
    specOneMode = "TARGET",
    specTwoMode = "TARGET",
};

--[[-------------------------------------------------------------------------------
    Methods
--]]-------------------------------------------------------------------------------

-- Print
function A:Print(text)
    print("|cff33ff99HaloMini:|r "..text);
end

-- Check if the player want Halo Minimalistic with the current spec
function A:HaloMiniWanted()
    if ( GetActiveSpecGroup() == 1 and A.db.specOne ) then
        return 1;
    end

    if ( GetActiveSpecGroup() == 2 and A.db.specTwo ) then
        return 1;
    end

    return nil;
end

-- Range check
-- 30651 Dertrok's First Wand -- 15 yards -- Harm
-- 10645 gnomish death ray -- 20 yards -- Harm
-- 24268 tailoring net (netherweave) -- 25 yards -- Harm
-- 10716 Gnomish Shrink Ray -- 30 yards -- Harm
-- 18904 Zorbin's Ultra-Shrinker -- 35 yards -- Harm

-- 1251 Linen Bandage -- 15 yards -- Help
-- 21519 mistletoe -- 20 yards -- Help
-- 56247 Box of Crossbow Bolts -- 25 yards -- Help
-- 1180 Scroll of Stamina -- 30 yards -- Help

-- CheckInteractDistance("unit", 1) -- 1 = inspect, 4 = follow -- both = 28 yards -- help & harm
function A:CheckRange()
    local r15, r20, r25, r30;

    if ( UnitIsFriend("player", A.modeUnit) ) then -- Friendly target
        r15 = IsItemInRange(1251, A.modeUnit);
        r20 = IsItemInRange(21519, A.modeUnit);
        r25 = IsItemInRange(56247, A.modeUnit);
        r30 = IsItemInRange(1180, A.modeUnit);
    else
        r15 = IsItemInRange(30651, A.modeUnit);
        r20 = IsItemInRange(10645, A.modeUnit);
        r25 = IsItemInRange(24268, A.modeUnit);
        r30 = IsItemInRange(10716, A.modeUnit);
    end

    -- Setting values to nil even if not in range and capable of using this item spell
    if ( r15 and r15 == 0 ) then r15 = nil; end
    if ( r20 and r20 == 0 ) then r20 = nil; end
    if ( r25 and r25 == 0 ) then r25 = nil; end
    if ( r30 and r30 == 0 ) then r30 = nil; end

    return r15, r20, r25, r30;
end

-- Set the check range mode
function A:SetMode()
    if ( GetActiveSpecGroup() == 1 ) then
        if ( A.db.specOneMode == "TARGET" ) then
            A.modeUnit = "target";
            A:EnableUpdate();
        elseif ( A.db.specOneMode == "MOUSEOVER" ) then
            A.modeUnit = "mouseover";
            A:EnableUpdate();
        elseif ( A.db.specOneMode == "BOTH" ) then
            A:EnableUpdateBoth();
        end
    elseif ( GetActiveSpecGroup() == 2 ) then
        if ( A.db.specTwoMode == "TARGET" ) then
            A.modeUnit = "target";
            A:EnableUpdate();
        elseif ( A.db.specTwoMode == "MOUSEOVER" ) then
            A.modeUnit = "mouseover";
            A:EnableUpdate();
        elseif ( A.db.specTwoMode == "BOTH" ) then
            A:EnableUpdateBoth();
        end
    end
end

-- Set the main frame scale
function A:SetScale()
    A.frame:SetScale(A.db.scale);
end

-- Set the main frame position
function A:SetPos()
    A.frame:ClearAllPoints();
    A.frame:SetPoint(A.db.point, A.db.relativeTo, A.db.relativePoint, A.db.offX, A.db.offY);
end

-- Reset frame position
function A:ResetPos()
    A.db.point = "CENTER";
    A.db.relativeTo = "UIParent";
    A.db.relativePoint = "CENTER";
    A.db.offX = 0;
    A.db.offY = 0;

    A:SetPos();
end

-- Lock the frame
function A:Lock()
    -- Lock it
    A.frame:SetMovable(nil);
    A.frame:EnableMouse(nil);
    A.frame:SetScript("OnDragStart", nil);
    A.frame:SetScript("OnDragStop", nil);
    A.frame:SetScript("OnEnter", nil);
    A.frame:SetScript("OnLeave", nil);

    -- Enable OnUpdate script
    A:SetMode();

    -- Tell the player the frame is lock
    A:Print(L["LOCKED"]);
end

-- Unlock the frame
function A:Unlock()
    -- Disable OnUpdate script
    A.frame:SetScript("OnUpdate", nil);

    -- Set movable
    A.frame:SetMovable(1);
    A.frame:EnableMouse(1);
    A.frame:RegisterForDrag("LeftButton");
    A.frame:SetScript("OnDragStart", A.frame.StartMoving);
    A.frame:SetScript("OnDragStop", function()
        A.frame:StopMovingOrSizing();

        local point, relativeTo, relativePoint, offX, offY = A.frame:GetPoint(1);

        A.db.point = point;
        A.db.relativeTo = relativeTo;
        A.db.relativePoint = relativePoint;
        A.db.offX = offX;
        A.db.offY = offY;
    end);
    A.frame:SetScript("OnEnter", function()
        A:DisplayUnlockedTooltip();
    end);
    A.frame:SetScript("OnLeave", function()
        GameTooltip:Hide();
    end);

    -- Display arrows and green dot
    A.frame.middleGray:Hide();
    A.frame.middleGreen:Hide();
    A.frame.middleYellow:Hide();
    A.frame.middleRed:Hide();
    A.frame.middleInfo:Show();
    A.frame.left:Show();
    A.frame.right:Show();

    -- Tell the player the frame is unlock
    A:Print(L["UNLOCKED"]);
end

-- Toggle lock/unlock
function A:ToggleLock()
    if ( A.frame:IsMovable() ) then
        A:Lock();
    else
        A:Unlock();
    end
end

-- Display a warning tooltip when hovering unlocked main frame
function A:DisplayUnlockedTooltip()
    if ( A.frame:GetRight() >= ( GetScreenWidth() / 2 ) ) then
        GameTooltip:SetOwner(A.frame, "ANCHOR_LEFT");
    else
        GameTooltip:SetOwner(A.frame, "ANCHOR_RIGHT");
    end

    GameTooltip:AddDoubleLine("|cffffffffHalo Minimalistic", "|cff33ff99v"..A.version);
    GameTooltip:AddLine(" ");
    GameTooltip:AddLine(L["TOOLTIP_UNLOCKED_INFO"], nil, nil, nil, 1);

    GameTooltip:Show();
end

-- Get a specializaion name
function A:GetSpecializationName(specNum)
    if ( GetSpecialization(false, false, specNum) ) then
        return select(2, GetSpecializationInfo(GetSpecialization(false, false, specNum), false, false));
    end

    return L["NONE"];
end

--[[-------------------------------------------------------------------------------
    Events
--]]-------------------------------------------------------------------------------

A:RegisterEvent("ADDON_LOADED");
A:RegisterEvent("PLAYER_LOGIN");

A:SetScript("OnEvent", function(self, event, ...)
    local arg1 = ...;

    if ( event == "ADDON_LOADED" and arg1 == "HaloMini" ) then
        -- Setting default savedvar if empty
        if ( not _G["haloMiniCharacterDB"] ) then _G["haloMiniCharacterDB"] = {}; end

        for k,v in pairs(A.defaultDB) do
            if ( type(_G["haloMiniCharacterDB"][k]) == "nil" ) then
                _G["haloMiniCharacterDB"][k] = v;
            end
        end

        -- Adding savedvar to local object
        A.db = _G["haloMiniCharacterDB"];

        -- Unregister event
        A:UnregisterEvent("ADDON_LOADED");
    elseif ( event == "PLAYER_LOGIN" ) then
        if ( select(3, UnitClass("player")) == 5 and UnitLevel("player") == 90 ) then -- It's a level 90 priest
            -- Setting frame and layers objects to local object
            A.frame = _G["HaloMiniFrame"];
            A.frame.middleGray = _G["HaloMiniFrameMiddleGray"];
            A.frame.middleGreen = _G["HaloMiniFrameMiddleGreen"];
            A.frame.middleYellow = _G["HaloMiniFrameMiddleYellow"];
            A.frame.middleRed = _G["HaloMiniFrameMiddleRed"];
            A.frame.middleInfo = _G["HaloMiniFrameMiddleInfo"];
            A.frame.left = _G["HaloMiniFrameLeft"];
            A.frame.right = _G["HaloMiniFrameRight"];

            -- Setting slash commands
            A:SetSlashCommands();

            -- Adding options panel to Blizzard interface options panel
            A:CreateOptionsPanel();

            -- On update value var
            A.frame.lastUpdate = 0;

            -- Starting main OnUpdate script (Frame is hidden atm), and setting mode
            A:SetMode();

            -- Registering events we only want to fire when everything is loaded
            A:RegisterEvent("PLAYER_ENTERING_WORLD");
            A:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
            A:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
            A:RegisterEvent("PET_BATTLE_OPENING_START");
            A:RegisterEvent("PET_BATTLE_CLOSE");

            -- Unregister event
            A:UnregisterEvent("PLAYER_LOGIN");
        else -- Not a priest
            A:Print(L["NOT_ON_PRIEST"]);
            A:UnregisterAllEvents();
            A = nil;
        end
    elseif ( event == "PLAYER_ENTERING_WORLD" ) then
        -- Setting options
        A:SetScale();
        A:SetPos();
        if ( not A.db.locked ) then A:Unlock(); end
        if ( A:HaloMiniWanted() ) then A.frame:Show(); end

        -- Unregistering event we only need it on login/reload
        A:UnregisterEvent("PLAYER_ENTERING_WORLD");
    elseif ( event == "ACTIVE_TALENT_GROUP_CHANGED" ) then
        -- Player switched specialisation, display Halo Minimalistic?
        if ( A:HaloMiniWanted() ) then
            A:SetMode();
            A.frame:Show();
        else
            A.frame:Hide();
        end
    elseif ( event == "PLAYER_SPECIALIZATION_CHANGED" ) then
        -- Fire when player learn/unlearn a specialization
        A.optionsPanel.frames.specOneEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_1_DESC"]:format(A:GetSpecializationName(1));
        A.optionsPanel.frames.specTwoEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_2_DESC"]:format(A:GetSpecializationName(2));
    elseif ( event == "PET_BATTLE_OPENING_START" ) then
        A.frame:Hide();
    elseif ( event == "PET_BATTLE_CLOSE" ) then
        if ( A:HaloMiniWanted() ) then A.frame:Show(); end
    end
end);

--[[-------------------------------------------------------------------------------
    OnUpdate scripts
--]]-------------------------------------------------------------------------------

-- Main OnUpdate script - Modes: target or mouseover
function A:EnableUpdate()
    A.frame:SetScript("OnUpdate", function(self, elapsed)
        self.lastUpdate = self.lastUpdate + elapsed;

        if ( self.lastUpdate > A.db.refreshRate ) then
            if ( UnitExists(A.modeUnit) ) then -- Got a target or mouseover
                -- Getting range status
                local r15, r20, r25, r30 = A:CheckRange();

                -- Setting layers
                if ( r15 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                elseif ( not r15 and r20 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r20 and r25 ) then
                    A.frame.left:Hide();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Show();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Hide();
                elseif ( not r25 and r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                end

                A.frame.middleGray:Hide();
            else -- No target or mouseover
                A.frame.middleGray:Show();
                A.frame.middleGreen:Hide();
                A.frame.middleYellow:Hide();
                A.frame.middleRed:Hide();
                A.frame.left:Hide();
                A.frame.right:Hide();
            end

            A.frame.middleInfo:Hide();
            self.lastUpdate = 0;
        end
    end);
end

-- Main OnUpdate script - Mode: Both
function A:EnableUpdateBoth()
    A.frame:SetScript("OnUpdate", function(self, elapsed)
        self.lastUpdate = self.lastUpdate + elapsed;

        if ( self.lastUpdate > A.db.refreshRate ) then
            if ( UnitExists("mouseover") ) then -- Got a mouseover
                A.modeUnit = "mouseover";

                -- Getting range status
                local r15, r20, r25, r30 = A:CheckRange();

                -- Setting layers
                if ( r15 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                elseif ( not r15 and r20 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r20 and r25 ) then
                    A.frame.left:Hide();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Show();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Hide();
                elseif ( not r25 and r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                end

                A.frame.middleGray:Hide();
            elseif ( UnitExists("target") ) then -- Got a target
                A.modeUnit = "target";

                -- Getting range status
                local r15, r20, r25, r30 = A:CheckRange();

                -- Setting layers
                if ( r15 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                elseif ( not r15 and r20 ) then
                    A.frame.left:Show();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r20 and r25 ) then
                    A.frame.left:Hide();
                    A.frame.right:Hide();
                    A.frame.middleGreen:Show();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Hide();
                elseif ( not r25 and r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Show();
                    A.frame.middleRed:Hide();
                elseif ( not r30 ) then
                    A.frame.left:Hide();
                    A.frame.right:Show();
                    A.frame.middleGreen:Hide();
                    A.frame.middleYellow:Hide();
                    A.frame.middleRed:Show();
                end

                A.frame.middleGray:Hide();
            else -- No target or mouseover
                A.frame.middleGray:Show();
                A.frame.middleGreen:Hide();
                A.frame.middleYellow:Hide();
                A.frame.middleRed:Hide();
                A.frame.left:Hide();
                A.frame.right:Hide();
            end

            A.frame.middleInfo:Hide();
            self.lastUpdate = 0;
        end
    end);
end

--[[-------------------------------------------------------------------------------
    Slash command
--]]-------------------------------------------------------------------------------

function A:SetSlashCommands()
    _G["SLASH_HALOMINI1"], _G["SLASH_HALOMINI2"] = "/hm", "/halomini";
    function SlashCmdList.HALOMINI(args)
        if ( args == "" ) then
            A:ToggleLock();
        elseif ( args == "wipedb" ) then
            for k,v in pairs(A.defaultDB) do
                A.db[k] = v;
            end

            A:Print(L["DATABASE_WIPED"]);
        elseif ( args == "config" ) then
            InterfaceOptionsFrame_OpenToCategory(A.optionsPanel);
        elseif ( args == "reset" ) then
            A:ResetPos();
        else
            A:Print(L["COMMAND_1"]);
            A:Print(L["COMMAND_2"]);
            A:Print(L["COMMAND_3"]);
            --A:Print(L["COMMAND_4"]);
        end
    end
end
