﻿-- ********************************************************************************
-- Halo Minimalistic
-- A minimalistic halo range display.
-- By: Shenton
--
-- Options.lua
-- ********************************************************************************

--[[-------------------------------------------------------------------------------
    Globals to locals
--]]-------------------------------------------------------------------------------

local _G = _G;
local format = format;
local tonumber = tonumber;
local string = string;
local CreateFrame = CreateFrame;
local InterfaceOptions_AddCategory = InterfaceOptions_AddCategory;
local PlaySound = PlaySound;

--[[-------------------------------------------------------------------------------
    Main object
--]]-------------------------------------------------------------------------------

local A = _G["HaloMinimalisticGlobal"];
local L = A.L;

--[[-------------------------------------------------------------------------------
    Options panel
--]]-------------------------------------------------------------------------------

function A:SpecOneDropdownClick()
    A.optionsPanel.frames.specOneDropDown:SetValue(self.value);
end

function A:SpecTwoDropdownClick()
    A.optionsPanel.frames.specTwoDropDown:SetValue(self.value);
end

function A:SpecOneDropdownInit()
    A:SpecDropdownInit(1);
end

function A:SpecTwoDropdownInit()
    A:SpecDropdownInit(2);
end

function A:SpecDropdownInit(spec)
    local func, selected;

    if ( spec == 1 ) then
        func = A.SpecOneDropdownClick;
        selected = A.db.specOneMode;
    else
        func = A.SpecTwoDropdownClick;
        selected = A.db.specTwoMode;
    end

    local info = UIDropDownMenu_CreateInfo();

    info.text = L["TARGET"];
    info.func = func;
    info.value = "TARGET";
    if ( info.value == selected ) then
        info.checked = 1;
    else
        info.checked = nil;
    end
    info.tooltipTitle = L["TARGET"];
    --info.tooltipText = ;
    UIDropDownMenu_AddButton(info);

    info.text = L["MOUSEOVER"];
    info.func = func;
    info.value = "MOUSEOVER";
    if ( info.value == selected ) then
        info.checked = 1;
    else
        info.checked = nil;
    end
    info.tooltipTitle = L["MOUSEOVER"];
    --info.tooltipText = ;
    UIDropDownMenu_AddButton(info);

    info.text = L["BOTH"];
    info.func = func;
    info.value = "BOTH";
    if ( info.value == selected ) then
        info.checked = 1;
    else
        info.checked = nil;
    end
    info.tooltipTitle = L["BOTH"];
    --info.tooltipText = ;
    UIDropDownMenu_AddButton(info);
end

function A:CreateOptionsPanel()
    -- Options panel frame
    local panel = CreateFrame("Frame", "HaloMiniOptionsPanel");
    panel.layers = {};
    panel.frames = {};

    -- Adding options panel to main object
    A.optionsPanel = panel;

    -- Tab category name
    panel.name = "Halo Minimalistic";

    -- Title
    panel.layers.title = panel:CreateFontString("HaloMiniOptionsPanelTitle", "ARTWORK", "GameFontNormalLarge");
    panel.layers.title:ClearAllPoints();
    panel.layers.title:SetPoint("TOPLEFT", panel, "TOPLEFT", 16, -16);
    panel.layers.title:SetText("Halo Minimalistic");

    -- Version
    panel.layers.version = panel:CreateFontString("HaloMiniOptionsPanelVersion", "ARTWORK", "GameFontNormalLarge");
    panel.layers.version:ClearAllPoints();
    panel.layers.version:SetPoint("TOPRIGHT", panel, "TOPRIGHT", -16, -16);
    panel.layers.version:SetText(L["OPTIONS_VERSION"]:format(A.version));

    -- Lock checkbox
    panel.frames.lock = CreateFrame("CheckButton", "HaloMiniOptionsPanelLock", panel, "InterfaceOptionsCheckButtonTemplate");
    panel.frames.lock:ClearAllPoints();
    panel.frames.lock:SetPoint("TOPLEFT", panel.layers.title, "BOTTOMLEFT", 0, -16);
    _G["HaloMiniOptionsPanelLockText"]:SetText(L["LOCKED"]);
    panel.frames.lock.tooltipText = L["LOCKED"];
    panel.frames.lock.tooltipRequirement = L["OPTIONS_LOCK_DESC"];
    panel.frames.lock:SetScript("OnShow", function(self)
        if ( A.db.locked ) then
            self:SetChecked(1);
        else
            self:SetChecked(nil);
        end
    end);
    panel.frames.lock:SetScript("OnClick", function(self)
        if ( self:GetChecked() ) then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end

        A.db.locked = not A.db.locked;

        if ( A.db.locked ) then
            A:Lock();
            self:SetChecked(1);
        else
            A:Unlock();
            self:SetChecked(nil);
        end
    end);

    -- Reset position button
    panel.frames.resetPos = CreateFrame("Button", "HaloMiniOptionsPanelResetPos", panel, "UIPanelButtonTemplate");
    panel.frames.resetPos:SetSize(130, 21);
    panel.frames.resetPos:ClearAllPoints();
    panel.frames.resetPos:SetPoint("LEFT", panel.frames.lock, "LEFT", 280, 0);
    _G["HaloMiniOptionsPanelResetPosText"]:SetText(L["RESET_POS"]);
    panel.frames.resetPos.tooltipText = L["OPTIONS_RESET_POS_DESC"];
    panel.frames.resetPos:SetScript("OnClick", function()
        A:ResetPos();
    end);

    -- Scale slider
    panel.frames.scale = CreateFrame("Slider", "HaloMiniOptionsPanelScale", panel, "OptionsSliderTemplate");
    panel.frames.scale:SetMinMaxValues(0.5, 3);
    panel.frames.scale:SetValueStep(0.1);
    _G["HaloMiniOptionsPanelScaleLow"]:SetText(0.5);
    _G["HaloMiniOptionsPanelScaleHigh"]:SetText(3);
    panel.frames.scale:ClearAllPoints();
    panel.frames.scale:SetPoint("TOPLEFT", panel.frames.lock, "BOTTOMLEFT", 0, -32);
    _G["HaloMiniOptionsPanelScaleText"]:SetText(L["SCALE"]);
    panel.frames.scale.tooltipText = L["SCALE"];
    panel.frames.scale.tooltipRequirement = L["OPTIONS_SCALE_DESC"];
    panel.frames.scale:SetScript("OnShow", function(self)
        self:SetValue(A.db.scale);
    end);
    panel.frames.scale:SetScript("OnValueChanged", function(self, value)
        A.db.scale = tonumber(string.format("%.1f", value));
        panel.layers.scaleValue:SetText(A.db.scale);
        A:SetScale();
    end);

    -- Scale slider value display
    panel.layers.scaleValue = panel:CreateFontString("HaloMiniOptionsPanelScaleValue", "ARTWORK", "GameFontHighlight");
    panel.layers.scaleValue:ClearAllPoints();
    panel.layers.scaleValue:SetPoint("TOP", panel.frames.scale, "BOTTOM", 0, -8);
    panel.layers.scaleValue:SetText(A.db.scale);

    -- Refresh rate slider
    panel.frames.refreshRate = CreateFrame("Slider", "HaloMiniOptionsPanelRefreshRate", panel, "OptionsSliderTemplate");
    panel.frames.refreshRate:SetMinMaxValues(0.1, 1);
    panel.frames.refreshRate:SetValueStep(0.1);
    _G["HaloMiniOptionsPanelRefreshRateLow"]:SetText(0.1);
    _G["HaloMiniOptionsPanelRefreshRateHigh"]:SetText(1);
    panel.frames.refreshRate:ClearAllPoints();
    panel.frames.refreshRate:SetPoint("TOPLEFT", panel.frames.scale, "BOTTOMLEFT", 0, -64);
    _G["HaloMiniOptionsPanelRefreshRateText"]:SetText(L["REFRESH_RATE"]);
    panel.frames.refreshRate.tooltipText = L["REFRESH_RATE"];
    panel.frames.refreshRate.tooltipRequirement = L["OPTIONS_REFRESH_RATE_DESC"];
    panel.frames.refreshRate:SetScript("OnShow", function(self)
        self:SetValue(A.db.refreshRate);
    end);
    panel.frames.refreshRate:SetScript("OnValueChanged", function(self, value)
        A.db.refreshRate = tonumber(string.format("%.1f", value));
        panel.layers.refreshRateValue:SetText(A.db.refreshRate);
    end);

    -- Refresh rate slider value display
    panel.layers.refreshRateValue = panel:CreateFontString("HaloMiniOptionsPanelRefreshRateValue", "ARTWORK", "GameFontHighlight");
    panel.layers.refreshRateValue:ClearAllPoints();
    panel.layers.refreshRateValue:SetPoint("TOP", panel.frames.refreshRate, "BOTTOM", 0, -8);
    panel.layers.refreshRateValue:SetText(A.db.refreshRate);

    -- Spec 1 text
    panel.layers.specOneText = panel:CreateFontString("HaloMiniOptionsPanelSpecOneText", "ARTWORK", "GameFontNormal");
    panel.layers.specOneText:ClearAllPoints();
    panel.layers.specOneText:SetPoint("TOPLEFT", panel.frames.refreshRate, "BOTTOMLEFT", 0, -48);
    panel.layers.specOneText:SetText(L["SPEC_1"]);

    -- Spec 1 background
    panel.layers.specOneBack = panel:CreateTexture("HaloMiniOptionsPanelSpecOneBack", "BACKGROUND");
    panel.layers.specOneBack:SetTexture(0.3, 0.3, 0.3, 0.5);
    panel.layers.specOneBack:SetSize(200, 100);
    panel.layers.specOneBack:ClearAllPoints();
    panel.layers.specOneBack:SetPoint("TOPLEFT", panel.layers.specOneText, "BOTTOMLEFT", 0, -8);

    -- Spec 1 enable checkbox
    panel.frames.specOneEnable = CreateFrame("CheckButton", "HaloMiniOptionsPanelSpecOneEnable", panel, "InterfaceOptionsCheckButtonTemplate");
    panel.frames.specOneEnable:ClearAllPoints();
    panel.frames.specOneEnable:SetPoint("TOPLEFT", panel.layers.specOneBack, "TOPLEFT", 0, 0);
    _G["HaloMiniOptionsPanelSpecOneEnableText"]:SetText(L["ENABLE"]);
    panel.frames.specOneEnable.tooltipText = L["ENABLE"];
    panel.frames.specOneEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_1_DESC"]:format(A:GetSpecializationName(1));
    panel.frames.specOneEnable:SetScript("OnShow", function(self)
        if ( A.db.specOne ) then
            self:SetChecked(1);
        else
            self:SetChecked(nil);
        end
    end);
    panel.frames.specOneEnable:SetScript("OnClick", function(self)
        if ( self:GetChecked() ) then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end

        A.db.specOne = not A.db.specOne;

        if ( A.db.specOne ) then
            if ( GetActiveSpecGroup() == 1 ) then
                A.frame:Show();
            end

            self:SetChecked(1);
        else
            if ( GetActiveSpecGroup() == 1 ) then
                A.frame:Hide();
            end

            self:SetChecked(nil);
        end
    end);

    -- Spec 1 dropdown text
    panel.layers.specOneDropdownText = panel:CreateFontString("HaloMiniOptionsPanelSpecOneDopdownText", "ARTWORK", "GameFontNormal");
    panel.layers.specOneDropdownText:ClearAllPoints();
    panel.layers.specOneDropdownText:SetPoint("TOPLEFT", panel.frames.specOneEnable, "BOTTOMLEFT", 5, -8);
    panel.layers.specOneDropdownText:SetText(L["MODE"]);

    -- Spec 1 dropdown
    panel.frames.specOneDropDown = CreateFrame("Frame", "HaloMiniOptionsPanelSpecOneDropdown", panel, "UIDropDownMenuTemplate");
    panel.frames.specOneDropDown:EnableMouse(1);
    panel.frames.specOneDropDown:ClearAllPoints();
    panel.frames.specOneDropDown:SetPoint("TOPLEFT", panel.layers.specOneDropdownText, "BOTTOMLEFT", -16, -5);
    panel.frames.specOneDropDown:SetScript("OnShow", function(self)
        if ( self.loaded ) then return; end

        self.value = A.db.specOneMode;

        UIDropDownMenu_SetWidth(self, 120);
        UIDropDownMenu_Initialize(self, A.SpecOneDropdownInit);
        UIDropDownMenu_SetSelectedValue(self, self.value);

        self.SetValue = function(self, value)
            self.value = value;
            UIDropDownMenu_SetSelectedValue(self, value);
            A.db.specOneMode = value;
            A:SetMode();
        end;
        self.GetValue = function(self)
            return UIDropDownMenu_GetSelectedValue(self);
        end;
        self.RefreshValue = function(self)
            UIDropDownMenu_Initialize(self, A.SpecOneDropdownInit);
            UIDropDownMenu_SetSelectedValue(self, self.value);
        end;

        self.loaded = 1;
    end);
    panel.frames.specOneDropDown:SetScript("OnEnter", function(self)
        GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
        GameTooltip:SetText(L["MODE_DESC"], nil, nil, nil, nil, 1);
    end);
    panel.frames.specOneDropDown:SetScript("OnLeave", function(self)
        GameTooltip:Hide();
    end);

    -- Spec 2 text
    panel.layers.specTwoText = panel:CreateFontString("HaloMiniOptionsPanelSpecTwoText", "ARTWORK", "GameFontNormal");
    panel.layers.specTwoText:ClearAllPoints();
    panel.layers.specTwoText:SetPoint("LEFT", panel.layers.specOneText, "RIGHT", 150, 0);
    panel.layers.specTwoText:SetText(L["SPEC_2"]);

    -- Spec 2 background
    panel.layers.specTwoBack = panel:CreateTexture("HaloMiniOptionsPanelSpecTwoBack", "BACKGROUND");
    panel.layers.specTwoBack:SetTexture(0.3, 0.3, 0.3, 0.5);
    panel.layers.specTwoBack:SetSize(200, 100);
    panel.layers.specTwoBack:ClearAllPoints();
    panel.layers.specTwoBack:SetPoint("TOPLEFT", panel.layers.specTwoText, "BOTTOMLEFT", 0, -8);

    -- Spec 2 enable checkbox
    panel.frames.specTwoEnable = CreateFrame("CheckButton", "HaloMiniOptionsPanelSpecTwoEnable", panel, "InterfaceOptionsCheckButtonTemplate");
    panel.frames.specTwoEnable:ClearAllPoints();
    panel.frames.specTwoEnable:SetPoint("TOPLEFT", panel.layers.specTwoBack, "TOPLEFT", 0, 0);
    _G["HaloMiniOptionsPanelSpecTwoEnableText"]:SetText(L["ENABLE"]);
    panel.frames.specTwoEnable.tooltipText = L["ENABLE"];
    panel.frames.specTwoEnable.tooltipRequirement = L["ENABLE_WITH_SPEC_2_DESC"]:format(A:GetSpecializationName(2));
    panel.frames.specTwoEnable:SetScript("OnShow", function(self)
        if ( A.db.specTwo ) then
            self:SetChecked(1);
        else
            self:SetChecked(nil);
        end
    end);
    panel.frames.specTwoEnable:SetScript("OnClick", function(self)
        if ( self:GetChecked() ) then
            PlaySound("igMainMenuOptionCheckBoxOn");
        else
            PlaySound("igMainMenuOptionCheckBoxOff");
        end

        A.db.specTwo = not A.db.specTwo;

        if ( A.db.specTwo ) then
            if ( GetActiveSpecGroup() == 2 ) then
                A.frame:Show();
            end

            self:SetChecked(1);
        else
            if ( GetActiveSpecGroup() == 2 ) then
                A.frame:Hide();
            end

            self:SetChecked(nil);
        end
    end);

    -- Spec 2 dropdown text
    panel.layers.specTwoDropdownText = panel:CreateFontString("HaloMiniOptionsPanelSpecTwoDopdownText", "ARTWORK", "GameFontNormal");
    panel.layers.specTwoDropdownText:ClearAllPoints();
    panel.layers.specTwoDropdownText:SetPoint("TOPLEFT", panel.frames.specTwoEnable, "BOTTOMLEFT", 5, -8);
    panel.layers.specTwoDropdownText:SetText(L["MODE"]);

    -- Spec 2 dropdown
    panel.frames.specTwoDropDown = CreateFrame("Frame", "HaloMiniOptionsPanelSpecTwoDropdown", panel, "UIDropDownMenuTemplate");
    panel.frames.specTwoDropDown:EnableMouse(1);
    panel.frames.specTwoDropDown:ClearAllPoints();
    panel.frames.specTwoDropDown:SetPoint("TOPLEFT", panel.layers.specTwoDropdownText, "BOTTOMLEFT", -16, -5);
    panel.frames.specTwoDropDown:SetScript("OnShow", function(self)
        if ( self.loaded ) then return; end

        self.value = A.db.specTwoMode;

        UIDropDownMenu_SetWidth(self, 120);
        UIDropDownMenu_Initialize(self, A.SpecTwoDropdownInit);
        UIDropDownMenu_SetSelectedValue(self, self.value);

        self.SetValue = function(self, value)
            self.value = value;
            UIDropDownMenu_SetSelectedValue(self, value);
            A.db.specTwoMode = value;
            A:SetMode();
        end;
        self.GetValue = function(self)
            return UIDropDownMenu_GetSelectedValue(self);
        end;
        self.RefreshValue = function(self)
            UIDropDownMenu_Initialize(self, A.SpecTwoDropdownInit);
            UIDropDownMenu_SetSelectedValue(self, self.value);
        end;

        self.loaded = 1;
    end);
    panel.frames.specTwoDropDown:SetScript("OnEnter", function(self)
        GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
        GameTooltip:SetText(L["MODE_DESC"], nil, nil, nil, nil, 1);
    end);
    panel.frames.specTwoDropDown:SetScript("OnLeave", function(self)
        GameTooltip:Hide();
    end);

    -- Add panel to Blizzard interface options frame
    InterfaceOptions_AddCategory(panel);
end
